@@ -1,18 +1,19 @@
 /*
  * Copyright (C) 2008 Red Hat, Inc.
  *
- * This library is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
- * by the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Author(s):
  * 	Behdad Esfahbod
@@ -23,12 +24,8 @@
 
 #include <glib.h>
 
-#include "bidiarrays.hh"
-
 G_BEGIN_DECLS
 
-#define VTE_UNISTR_START 0x80000000
-
 /**
  * vteunistr:
  *
@@ -77,7 +74,7 @@ gunichar
 _vte_unistr_get_base (vteunistr s);
 
 /**
- * _vte_unistr_replace_base:
+ * _vte_unistr_append_to_string:
  * @s: a #vteunistr
  * @c: Unicode character to replace the base character of @s.
  *
@@ -89,47 +86,6 @@ _vte_unistr_get_base (vteunistr s);
 vteunistr
 _vte_unistr_replace_base (vteunistr s, gunichar c);
 
-static inline int
-_vte_g_string_append_unichar (GString *s, gunichar c)
-{
-        char outbuf[8];
-        guint len = 0;
-        int first;
-        int i;
-
-        if (c < 0x80) {
-                first = 0;
-                len = 1;
-        }
-        else if (c < 0x800) {
-                first = 0xc0;
-                len = 2;
-        }
-        else if (c < 0x10000) {
-                first = 0xe0;
-                len = 3;
-        }
-        else if (c < 0x200000) {
-                first = 0xf0;
-                len = 4;
-        }
-        else {
-                g_assert_not_reached ();
-        }
-
-        for (i = len - 1; i > 0; --i) {
-                outbuf[i] = (c & 0x3f) | 0x80;
-                c >>= 6;
-        }
-
-        outbuf[0] = c | first;
-
-        // GLib can do an inlined append()
-        g_string_append_len (s, outbuf, len);
-
-        return len;
-}
-
 /**
  * _vte_unistr_append_to_string:
  * @s: a #vteunistr
@@ -140,23 +96,16 @@ _vte_g_string_append_unichar (GString *s, gunichar c)
  **/
 void
 _vte_unistr_append_to_string (vteunistr s, GString *gs);
-#define _vte_unistr_append_to_string(s,gs)                              \
-        G_STMT_START {                                                  \
-                if G_LIKELY (s < VTE_UNISTR_START)                      \
-                        _vte_g_string_append_unichar (gs, (gunichar)s); \
-                else                                                    \
-                        (_vte_unistr_append_to_string) (s, gs);         \
-        } G_STMT_END
 
 /**
  * _vte_unistr_append_to_gunichars:
  * @s: a #vteunistr
- * @a: a #VteBidiChars of #gunichar items to append @s to
+ * @a: a #GArray of #gunichar items to append @s to
  *
  * Appends @s to @a.
  **/
 void
-_vte_unistr_append_to_gunichars (vteunistr s, VteBidiChars *a);
+_vte_unistr_append_to_gunichars (vteunistr s, GArray *a);
 
 /**
  * _vte_unistr_strlen:
@@ -168,8 +117,6 @@ _vte_unistr_append_to_gunichars (vteunistr s, VteBidiChars *a);
  **/
 int
 _vte_unistr_strlen (vteunistr s);
-#define _vte_unistr_strlen(s) \
-        ((s) < VTE_UNISTR_START ? 1 : (_vte_unistr_strlen)(s))
 
 G_END_DECLS
 
